@@ -1,3 +1,14 @@
+--------------------------------------------------------------------------
+-- This class is responsible for actively managing the actions of
+-- Lmod.  It is responsible finding and loading or unloading a
+-- modulefile.  It is also responsible for reloading modules when
+-- the module path changes.  Finally it is responsible for finding
+-- modulefiles for "avail"
+--
+-- @classmod Master
+
+require("strict")
+
 --------------------------------------------------------------------------
 -- Lmod License
 --------------------------------------------------------------------------
@@ -32,14 +43,7 @@
 --
 --------------------------------------------------------------------------
 
---------------------------------------------------------------------------
--- Master: This class is responsible for actively managing the actions of
---         Lmod.  It is responsible finding and loading or unloading a
---         modulefile.  It is also responsible for reloading modules when
---         the module path changes.  Finally it is responsible for finding
---         modulefiles for "avail"
 
-require("strict")
 local concatTbl          = table.concat
 local getenv             = os.getenv
 local sort               = table.sort
@@ -49,10 +53,8 @@ local removeEntry        = table.remove
 require("TermWidth")
 require("fileOps")
 require("string_utils")
-require("fillWords")
 require("loadModuleFile")
 require("utils")
-require("caseIndependent")
 
 local BeautifulTbl = require('BeautifulTbl')
 local ColumnTable  = require('ColumnTable')
@@ -71,7 +73,7 @@ local pack         = (_VERSION == "Lua 5.1") and argsPack   or table.pack
 local load         = (_VERSION == "Lua 5.1") and loadstring or load
 
 ------------------------------------------------------------------------
--- Master:new() a private ctor that is used to construct a singleton.
+-- a private ctor that is used to construct a singleton.
 
 s_master = {}
 
@@ -89,10 +91,10 @@ local numSearch     = 4
 local numSrchLatest = 2
 
 --------------------------------------------------------------------------
--- followDefault(): This local function is used to find a default file
---                  that maybe in symbolic link chain. This returns
---                  the absolute path.
-
+-- This local function is used to find a default file
+-- that maybe in symbolic link chain. This returns
+-- the absolute path.
+-- @param path A directory path
 local function followDefault(path)
    if (path == nil) then return nil end
    dbg.start{"followDefault(path=\"",path,"\")"}
@@ -127,18 +129,16 @@ local function followDefault(path)
 end
 
 --------------------------------------------------------------------------
--- find_inherit_module(): This local function is very similar to
---                        [[find_module_file]].  The idea is that
---                        an advanced user wants to inherit a compiler
---                        module and/or an mpi stack module.  They want
---                        to inherit the system compiler but add to the
---                        MODULEPATH.  This routine does a similar search
---                        for the module.  It searches for the original
---                        module in pathA and then searches again.  Only
---                        after finding the same named module does it
---                        return.
-
-
+-- This local function is very similar to
+-- [[find_module_file]].  The idea is that
+-- an advanced user wants to inherit a compiler
+-- module and/or an mpi stack module.  They want
+-- to inherit the system compiler but add to the
+-- MODULEPATH.  This routine does a similar search
+-- for the module.  It searches for the original
+-- module in pathA and then searches again.  Only
+-- after finding the same named module does it
+-- return.
 local function find_inherit_module(fullModuleName, oldFn)
    dbg.start{"find_inherit_module(",fullModuleName,",",oldFn, ")"}
 
@@ -197,9 +197,9 @@ local function find_inherit_module(fullModuleName, oldFn)
 end
 
 --------------------------------------------------------------------------
--- registerLoaded(): This function marks a module name as loaded and saves
---                   it to LOADEDMODULES and _LMFILES_.   This is only for
---                   compatibility with TCL/C module.
+-- This function marks a module name as loaded and saves
+-- it to LOADEDMODULES and _LMFILES_.   This is only for
+-- compatibility with TCL/C module.
 local function registerLoaded(full, fn)
    local modList  = "LOADEDMODULES"
    local modFn    = "_LMFILES_"
@@ -221,9 +221,9 @@ local function registerLoaded(full, fn)
 end
 
 --------------------------------------------------------------------------
--- registerUnloaded(): This function marks a module name as unloaded and 
---                     saves it to LOADEDMODULES and _LMFILES_.   This is
---                     only for compatibility with TCL/C module.
+-- This function marks a module name as unloaded and 
+-- saves it to LOADEDMODULES and _LMFILES_.   This is
+-- only for compatibility with TCL/C module.
 local function registerUnloaded(full, fn)
    local modList  = "LOADEDMODULES"
    local modFn    = "_LMFILES_"
@@ -245,8 +245,9 @@ local function registerUnloaded(full, fn)
 end
 
 --------------------------------------------------------------------------
--- Master:master() - Singleton Ctor.
-
+-- Singleton Ctor.
+-- @param self A Master object.
+-- @param safe A flag.
 function M.master(self, safe)
    dbg.start{"Master:master(safe: ",safe,")"}
    if (next(s_master) == nil) then
@@ -259,13 +260,13 @@ end
 
 
 --------------------------------------------------------------------------
--- access_find_module_file(): This local function returns the name of the
---                            filename and the full name of the module file.
---                            If the user gives the short name and it is
---                            loaded then that version is used not the default
---                            module file for the one named.  Otherwise
---                            find_module_file is used.
-
+-- This local function returns the name of the
+-- filename and the full name of the module file.
+-- If the user gives the short name and it is
+-- loaded then that version is used not the default
+-- module file for the one named.  Otherwise
+-- find_module_file is used.
+-- @param mname A MName object
 local function access_find_module_file(mname)
    local mt    = MT:mt()
    local sn    = mname:sn()
@@ -281,16 +282,17 @@ local function access_find_module_file(mname)
 end
 
 --------------------------------------------------------------------------
--- Master:access():  This member function is engine that runs the user
---                   commands "help", "whatis" and "show".  In each case
---                   mcp is set to MC_Access, MC_Access and MC_Show,
---                   respectively.  Using that value of mcp, the
---                   modulefile is found and evaluated by loadModuleFile.
---                   This causes the help, or whatis or showing the
---                   modulefile as the user requested.
-
+-- This member function is engine that runs the user
+-- commands "help", "whatis" and "show".  In each case
+-- mcp is set to MC_Access, MC_Access and MC_Show,
+-- respectively.  Using that value of mcp, the
+-- modulefile is found and evaluated by loadModuleFile.
+-- This causes the help, or whatis or showing the
+-- modulefile as the user requested.
+-- @param self A Master object.
 function M.access(self, ...)
    local masterTbl = masterTbl()
+   local shell     = s_master.shell
    local mt        = MT:mt()
    local mStack    = ModuleStack:moduleStack()
    local prtHdr    = systemG.prtHdr
@@ -298,7 +300,7 @@ function M.access(self, ...)
    local shellN    = s_master.shell:name()
    local help      = (systemG.help ~= dbg.quiet) and "-h" or nil
    local result, t
-   io.stderr:write("\n")
+   local A         = ShowResultsA
 
    local arg = pack(...)
    for i = 1, arg.n do
@@ -308,12 +310,13 @@ function M.access(self, ...)
       systemG.ModuleFn   = fn
       systemG.ModuleName = full
       if (fn and isFile(fn)) then
-         prtHdr()
+         A[#A+1] = prtHdr()
+         dbg.print{"RTM(2) #A:",#A,"\n"}
          if (masterTbl.rawDisplay) then
             local f     = io.open(fn, "r")
             local whole = f:read("*all")
-            io.stderr:write(whole)
             f:close()
+            A[#A+1]     = whole
          else
             mStack:push(full, moduleName, mname:sn(), fn)
             
@@ -322,14 +325,19 @@ function M.access(self, ...)
             loadModuleFile{file=fn,help=help, shell=shellN, mList = mList,
                         reportErr=true}
             mStack:pop()
-            io.stderr:write("\n")
+            A[#A+1] = "\n"
+            dbg.print{"RTM(3) #A:",#A,"\n"}
          end
       else
          a[#a+1] = moduleName
       end
    end
 
+   shell:echo(concatTbl(A,""))
+
+
    if (#a > 0) then
+      setWarningFlag()
       io.stderr:write("Failed to find the following module(s):  \"",
                       concatTbl(a,"\", \""),"\" in your MODULEPATH\n")
       io.stderr:write("Try: \n",
@@ -340,11 +348,10 @@ function M.access(self, ...)
 end
 
 --------------------------------------------------------------------------
--- Master:inheritModule(): load the module that has the same name as the
---                         one on the top of mStack.  This way a user
---                         can "inherit" the contents of a system module
---                         instead of copying.
-
+-- Load the module that has the same name as the
+-- one on the top of mStack.  This way a user
+-- can "inherit" the contents of a system module
+-- instead of copying.
 function M.inheritModule()
    dbg.start{"Master:inherit()"}
 
@@ -389,9 +396,10 @@ function M.inheritModule()
 end
 
 --------------------------------------------------------------------------
--- Master:load(): Load all requested modules.  Each module is unloaded
---                if it is currently loaded.
-
+-- Load all requested modules.  Each module is unloaded
+-- if it is currently loaded.
+-- @param mA An array of MName objects.
+-- @return An array of true/false values indicating success or not.
 function M.load(mA)
    local mStack = ModuleStack:moduleStack()
    local shellN = s_master.shell:name()
@@ -408,7 +416,7 @@ function M.load(mA)
       local loaded     = false
       local t          = mname:find()
       local fn         = t.fn
-      dbg.print{"fn: ", t.fn,"\n"}
+      dbg.print{"Master:load: i: ",i,", sn: ", sn, ", fn: ", t.fn,"\n"}
       if (mt:have(sn,"active")) then
          dbg.print{"Master:load reload module: \"",moduleName,
                    "\" as it is already loaded\n"}
@@ -429,34 +437,51 @@ function M.load(mA)
 	 mt:beginOP()
          mStack:push(t.modFullName, moduleName, sn, fn)
 	 loadModuleFile{file=fn, shell = shellN, mList = mList, reportErr=true}
+         mt:setStatus(sn, "active")
+         hook.apply("load",t)
          mStack:pop()
 	 mt:endOP()
          dbg.print{"Making ", t.modName, " active\n"}
-         mt:setStatus(sn, "active")
          registerLoaded(t.modFullName, fn)
          dbg.print{"Marked: ",t.modFullName," as loaded\n"}
          loaded = true
-         hook.apply("load",t)
       end
       a[#a+1] = loaded
+      if (not mcp.familyStackEmpty()) then
+         local b = {}
+         while (not mcp.familyStackEmpty()) do
+            local   b_old, b_new = mcp.familyStackPop()
+            LmodMessage("\nLmod is automatically replacing \"", b_old.fullName,
+                        "\" with \"", b_new.fullName, "\"\n" )
+            local umA   = {MName:new("mt",   b_old.sn) , MName:new("mt",   b_new.sn) }
+            local lmA   = {MName:new("load", b_new.usrName)}
+            b[#b+1]     = {umA = umA, lmA = lmA}
+         end
+
+         for i = 1,#b do
+            mcp:unload_usr(b[i].umA)
+            mcp:load(b[i].lmA)
+         end
+      end
    end
 
+
    if (M.safeToUpdate() and mt:safeToCheckZombies() and mStack:empty()) then
       dbg.print{"Master:load calling reloadAll()\n"}
       M.reloadAll()
    end
+
    dbg.fini("Master:load")
    return a
 end
 
 --------------------------------------------------------------------------
--- Master:refresh() - Loop over all active modules and reload each one.
---                    Since only the "shell" functions are active and all
---                    other Lmod functions are inactive because mcp is now
---                    MC_Refresh, there is no need to unload and reload the
---                    modulefiles.  Just call loadModuleFile() to redefine
---                    the aliases/shell functions in a subshell.
-
+-- Loop over all active modules and reload each one.
+-- Since only the "shell" functions are active and all
+-- other Lmod functions are inactive because mcp is now
+-- MC_Refresh, there is no need to unload and reload the
+-- modulefiles.  Just call loadModuleFile() to redefine
+-- the aliases/shell functions in a subshell.
 function M.refresh()
    local mStack  = ModuleStack:moduleStack()
    local mt      = MT:mt()
@@ -488,13 +513,12 @@ function M.refresh()
 end
 
 --------------------------------------------------------------------------
--- Master:reloadAll(): Loop over all modules in MT to see if they still
---                     can be seen.  We check every active module to see
---                     if the file associated with loaded module is the
---                     same as [[find_module_file()]] reports.  If not
---                     then it is unloaded and an attempt is made to reload
---                     it.  Each inactive module is re-loaded if possible.
-
+-- Loop over all modules in MT to see if they still
+-- can be seen.  We check every active module to see
+-- if the file associated with loaded module is the
+-- same as [[find_module_file()]] reports.  If not
+-- then it is unloaded and an attempt is made to reload
+-- it.  Each inactive module is re-loaded if possible.
 function M.reloadAll()
    local mt   = MT:mt()
    dbg.start{"Master:reloadAll()"}
@@ -566,16 +590,16 @@ function M.reloadAll()
 end
 
 --------------------------------------------------------------------------
--- Master:safeToUpdate() - [[safe]] is set during ctor. It is controlled
---                         by the command table in lmod.
-
+-- *safe* is set during ctor. It is controlled by the command table in lmod.
+-- @return the internal safe flag.
 function M.safeToUpdate()
    return s_master.safe
 end
 
 --------------------------------------------------------------------------
--- Master:unload() - unload modulefile(s) via the module names.
-
+-- Unload modulefile(s) via the module names.
+-- @param mA An array of MName objects.
+-- @return An array of true/false values indicating success or not.
 function M.unload(mA)
    local mStack = ModuleStack:moduleStack()
    local mt     = MT:mt()
@@ -619,8 +643,10 @@ function M.unload(mA)
          mt:endOP()
          dbg.print{"changePATH: ", mt._changePATHCount, "\n"}
          dbg.print{"calling mt:remove(\"",sn,"\")\n"}
+         local t = {fn = mt:fileName(sn), modFullName = mt:fullName(sn) }
          mt:remove(sn)
          registerUnloaded(fullModuleName, f)
+         hook.apply("unload",t)
          a[#a + 1] = true
       else
          a[#a + 1] = false
@@ -642,6 +668,10 @@ function M.unload(mA)
 end
 
 
+--------------------------------------------------------------------------
+-- Once the purge or unload happens, the sticky modules are reloaded.
+-- @param self A Master object
+-- @param force If true then don't reload.
 function M.reload_sticky(self, force)
 
    dbg.start{"Master:reload_sticky()"}
@@ -703,9 +733,11 @@ end
 
 
 --------------------------------------------------------------------------
--- findDefault(): Find the default module for the current directory
---                [[mpath]].
-
+-- Find the default module for the current directory *mpath*.
+-- @param mpath A single directory that is part of MODULEPATH
+-- @param sn The short name.
+-- @param versionA An array of versions for *sn*.
+-- @return A table describing the default.
 local function findDefault(mpath, sn, versionA)
    dbg.start{"Master.findDefault(mpath=\"",mpath,"\", "," sn=\"",sn,"\")"}
    local mt   = MT:mt()
@@ -756,22 +788,34 @@ local function findDefault(mpath, sn, versionA)
 end
 
 --------------------------------------------------------------------------
---  availEntry(): This routine is handed a single entry.  It check to see
---                if matches the search criteria.  It also adds any
---                properties such as default or anything from [[propT]].
-
-local function availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
+-- This routine is handed a single entry.  It check to see
+-- if matches the search criteria.  It also adds any
+-- properties such as default or anything from [[propT]].
+-- @param defaultOnly
+-- @param terse
+-- @param label
+-- @param szA
+-- @param searchA
+-- @param sn
+-- @param name
+-- @param f
+-- @param defaultModuleT
+-- @param dbT
+-- @param legendT
+-- @param a
+local function availEntry(defaultOnly, terse, label, szA, searchA, sn, name,
                           f, defaultModuleT, dbT, legendT, a)
-   dbg.start{"Master:availEntry(defaultOnly, terse, mpath, szA, searchA, "..
+   dbg.start{"Master:availEntry(defaultOnly, terse, label, szA, searchA, "..
                                 "sn, name, f, defaultModuleT, dbT, legendT, a)"}
 
    dbg.print{"sn:" ,sn, ", name: ", name,", defaultOnly: ",defaultOnly,
              ", szA: ",szA,"\n"}
-   local dflt     = ""
-   local sCount   = #searchA
-   local found    = false
-   local localdir = true
-   local mt       = MT:mt()
+   local dflt        = ""
+   local sCount      = #searchA
+   local found       = false
+   local localdir    = true
+   local hidden      = not masterTbl().show_hidden
+   local mt          = MT:mt()
 
    if (sCount == 0) then
       found = true
@@ -782,7 +826,7 @@ local function availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
             found = true
             break
          end
-         if (LMOD_MPATH_AVAIL ~= "no" and mpath:find(s)) then
+         if (LMOD_MPATH_AVAIL ~= "no" and label:find(s)) then
            found = true
            break
          end
@@ -791,13 +835,19 @@ local function availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
 
    local f_orig = f
    local d, bn  = splitFileName(f)
-   f            = pathJoin(abspath(d),bn)
+   local d2     = abspath(d)
+   f            = pathJoin(d2,bn)
+   local fabs   = abspath_localdir(f)
 
    dbg.print{"defaultOnly: ",defaultOnly, ", defaultModuleT.fn: ",defaultModuleT.fn,
-             ", f_orig: ",f_orig,", f: ", f, "\n"}
+             ", f_orig: ",f_orig,", f: ", f,", d: ", d,", d2: ", d2, ", fabs: ",fabs,"\n"}
+
+   local isDefault = (defaultModuleT.fn == f_orig or defaultModuleT.fn == fabs)
+
+   dbg.print{"isDefault: ",isDefault, "\n"}
 
    --if (defaultOnly and defaultModuleT.fn ~= abspath(f, localdir)) then
-   if (defaultOnly and defaultModuleT.fn ~= f) then
+   if (defaultOnly and not isDefault ) then
       found = false
    end
 
@@ -809,14 +859,13 @@ local function availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
 
    local mname   = MName:new("load", name)
    local version = mname:version() or ""
-   if (version:sub(1,1) == "." or sn:sub(1,1) == ".") then
+   if (hidden and version:sub(1,1) == "." or sn:sub(1,1) == ".") then
       dbg.print{"Not printing a dot modulefile\n"}
       dbg.fini("Master:availEntry")
       return
    end
 
 
-
    if (terse) then
       -- Print out directory (e.g. gcc) for tab-completion
       -- But only print it out when reporting defaultOnly.
@@ -830,15 +879,14 @@ local function availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
    else
       dbg.print{"defaultModuleT.fn: ",defaultModuleT.fn,
                 ", kind: ", defaultModuleT.kind,
-                ", num: ",  defaultModuleT.num,
+                ", num: ",  defaultModuleT.num > 1,
                 ", f_orig: ", f_orig,
                 ", f: ",f,
                 "\n"}
 
 
       --if ((defaultModuleT.fn == abspath(f, localdir)) and
-      if ((defaultModuleT.fn == f) and
-          (defaultModuleT.num > 1) and not defaultOnly ) then
+      if ((isDefault) and (defaultModuleT.num > 1) and not defaultOnly ) then
          dflt = Default
          legendT[Default] = "Default Module"
       end
@@ -868,20 +916,23 @@ end
 
 
 ---------------------------------------------------------------------------
--- availDir(): Handle a single directory and collect any entries through
---             [[availEntry]].
-
-
-local function availDir(defaultOnly, terse, searchA, mpath, locationT, availT,
+-- Handle a single directory and collect any entries through *availEntry*
+-- @param defaultOnly
+-- @param terse
+-- @param searchA
+-- @param label
+-- @param locationT
+-- @param availT
+-- @param dbT
+-- @param a
+-- @param legendT
+local function availDir(defaultOnly, terse, searchA, label, locationT, availT,
                         dbT, a, legendT)
    dbg.start{"Master.availDir(defaultOnly= ",defaultOnly,", terse= ",terse,
-             ", searchA=(",concatTbl(searchA,", "), "), mpath= \"",mpath,"\", ",
+             ", searchA=(",concatTbl(searchA,", "), "), label= \"",label,"\", ",
              ",locationT, availT, dbT, a, legendT)"}
-   local attr    = lfs.attributes(mpath)
-   local mt      = MT:mt()
-   if (not attr or type(attr) ~= "table" or attr.mode ~= "directory"
-       or not posix.access(mpath,"x")) then
-      dbg.print{"Skipping non-existant directory: ", mpath,"\n"}
+   if (availT == nil) then
+      dbg.print{"(1) Skipping non-existant directory: ", label,"\n"}
       dbg.fini("Master.availDir")
       return
    end
@@ -893,21 +944,22 @@ local function availDir(defaultOnly, terse, searchA, mpath, locationT, availT,
       dbg.print{"sn: ",sn,"\n"}
       local defaultModuleT = locationT[sn].default
       local aa             = {}
-      local szA            = #versionA
-      if (szA == 0) then
-         local fn = versionA[0].file
-         availEntry(defaultOnly, terse, mpath, szA, searchA, sn, sn, fn,
+      local szA            = versionA.total - versionA.hidden
+      if (szA == 0 and versionA.hidden == 0) then
+         local fn    = versionA[0].fn
+         dbg.print{"fn: ",fn,"\n"}
+         availEntry(defaultOnly, terse, label, szA, searchA, sn, sn, fn,
                     defaultModuleT, dbT, legendT, a)
       else
-         if (terse) then
+         if (terse and szA > 0) then
             -- Print out directory (e.g. gcc) for tab-completion
-            availEntry(defaultOnly, terse, mpath, szA, searchA, sn, sn, "",
+            availEntry(defaultOnly, terse, label, szA, searchA, sn, sn, "",
                        defaultModuleT, dbT, legendT, a)
          end
          for i = 1, #versionA do
             local name = pathJoin(sn, versionA[i].version)
-            local f    = versionA[i].file
-            availEntry(defaultOnly, terse, mpath, szA, searchA, sn, name,
+            local f    = versionA[i].fn
+            availEntry(defaultOnly, terse, label, szA, searchA, sn, name,
                        f, defaultModuleT, dbT, legendT, a)
          end
       end
@@ -916,11 +968,11 @@ local function availDir(defaultOnly, terse, searchA, mpath, locationT, availT,
 end
 
 --------------------------------------------------------------------------
--- availOptions(): "module avail " takes options that are given here.
---                 Note that these are also available with the Lmod
---                 options. So "module -t avail" and "module avail -t" are
---                 the same thing.
-
+-- "module avail " takes options that are given here.
+-- Note that these are also available with the Lmod
+-- options. So "module -t avail" and "module avail -t" are
+-- the same thing.
+-- @param argA
 local function availOptions(argA)
    local usage = "Usage: module avail [options] search1 search2 ..."
    local cmdlineParser = Optiks:new{usage=usage, version=""}
@@ -943,44 +995,115 @@ local function availOptions(argA)
 end
 
 --------------------------------------------------------------------------
--- Master:avail(): Report the available modules with properties and
---                 defaults.  Run results through pager.
+--
+-- @param availStyle
+-- @param mpathA
+-- @param availT
+local function regroup_avail_blocks(availStyle, mpathA, availT)
+   --------------------------------------------------------------------------
+   -- call hook to see if site wants to relabel and re-organize avail layout
+
+   if (availStyle == "system") then
+      return mpathA, availT
+   end
+
+
+   local labelT    = {}
+
+   for i = 1, #mpathA do
+      local mpath = mpathA[i]
+      if ( availT[mpath] ~= nil) then
+         labelT[mpath] = mpath
+      end
+   end
+
+   hook.apply("avail",labelT)
+
+   local label2mpathT = {}
+
+   for i = 1, #mpathA do
+      local mpath = mpathA[i]
+      if ( availT[mpath] ~= nil) then
+         local label = labelT[mpath]
+         local a     = label2mpathT[label] or {}
+         a[#a+1]     = i
+         label2mpathT[label] = a
+      end
+   end
+
+   if (dbg.active()) then
+      for label, vA in pairs(label2mpathT) do
+         dbg.print{"label: ",label,":",}
+         for i = 1, #vA do
+            io.stderr:write(" ",tostring(vA[i]))
+         end
+         io.stderr:write("\n")
+      end
+   end
+
+   local orderA = {}
+   for label, vA in pairs(label2mpathT) do
+      orderA[#orderA + 1] = {vA[1], label}
+   end
+   sort(orderA, function(a,b) return a[1] < b[1] end )
+
+   if (dbg.active()) then
+      for j = 1, #orderA do
+         dbg.print{j,", orderA: idx: ",orderA[j][1],", label: ",orderA[j][2],"\n"}
+      end
+   end
+
+   local availNT = {}
+   local labelA  = {}
+   for j = 1, #orderA do
+      local label    = orderA[j][2]
+      local a        = label2mpathT[label]
+      labelA[j]      = label
+      availNT[label] = {}
+      for i = 1, #a do
+         local mpath = mpathA[a[i]]
+         for k,v in pairs(availT[mpath]) do
+            if (availNT[label][k] == nil) then
+               availNT[label][k] = v
+            else
+               local vA = availNT[label][k]
+               for iv = 1,#v do
+                  vA[#vA+1] = v[iv]
+               end
+               sort(vA, function(a,b) return a.parseV < b.parseV end)
+            end
+         end
+      end
+   end
+   return labelA, availNT
+end
+--------------------------------------------------------------------------
+-- Report the available modules with properties and
+-- defaults.  Run results through pager.
+-- @param argA User requested search terms.
 function M.avail(argA)
    dbg.start{"Master.avail(",concatTbl(argA,", "),")"}
    local mt        = MT:mt()
+   local shell     = s_master.shell
    local mpathA    = mt:module_pathA()
    local twidth    = TermWidth()
    local masterTbl = masterTbl()
 
-   local cache     = Cache:cache{quiet = masterTbl.terse}
-   local moduleT   = cache:build()
-   local dbT       = {}
-
    local optionTbl, searchA = availOptions(argA)
    if (not masterTbl.regexp) then
       for i = 1, #searchA do
-         searchA[i] = caseIndependent(searchA[i])
+         searchA[i] = searchA[i]:caseIndependent()
       end
    end
    local defaultOnly = optionTbl.defaultOnly or masterTbl.defaultOnly
    local terse       = optionTbl.terse       or masterTbl.terse
-
-   local baseMpath = mt:getBaseMPATH()
-   if (not terse and (baseMpath == nil or baseMpath == '' or next(moduleT) == nil)) then
-     LmodError("avail is not possible, MODULEPATH is not set or not set with valid paths.\n")
-   end
-
-   local spider    = Spider:new()
-   spider:buildSpiderDB({"default"}, moduleT, dbT)
-
-   local legendT   = {}
-   local availT    = mt:availT()
-   local locationT = mt:locationTbl()
-
-   local aa        = {}
+   local legendT     = {}
+   local dbT         = {}
 
    if (terse) then
       dbg.print{"doing --terse\n"}
+      local availT    = mt:availT()
+      local locationT = mt:locationTbl()
       for ii = 1, #mpathA do
          local mpath = mpathA[ii]
          local a     = {}
@@ -996,12 +1119,36 @@ function M.avail(argA)
       return
    end
 
-   for _,mpath in ipairs(mpathA) do
-      local a = {}
-      availDir(defaultOnly, terse, searchA, mpath, locationT, availT[mpath], dbT, a, legendT)
+   local cache     = Cache:cache{quiet = masterTbl.terse}
+   local moduleT   = cache:build()
+
+   local baseMpath = mt:getBaseMPATH()
+   if (not terse and (baseMpath == nil or baseMpath == '' or next(moduleT) == nil)) then
+     LmodError("avail is not possible, MODULEPATH is not set or not set with valid paths.\n")
+   end
+
+   local spider     = Spider:new()
+   spider:buildSpiderDB({"default"}, moduleT, dbT)
+
+   local labelA     = false
+   local availT     = mt:availT()
+   local locationT  = mt:locationTbl()
+   local availStyle = masterTbl.availStyle
+
+   --------------------------------------------------------------------------
+   -- call hook to see if site wants to relabel and re-organize avail layout
+
+   labelA, availT  = regroup_avail_blocks(availStyle, mpathA, availT)
+
+   local aa        = {}
+
+   for j = 1, #labelA do
+      local a     = {}
+      local label = labelA[j]
+      availDir(defaultOnly, terse, searchA, label, locationT, availT[label], dbT, a, legendT)
       if (next(a)) then
          aa[#aa+1] = "\n"
-         aa[#aa+1] = banner:bannerStr(mpath)
+         aa[#aa+1] = banner:bannerStr(label)
          aa[#aa+1] = "\n"
          local ct  = ColumnTable:new{tbl=a, gap=1, len=length}
          aa[#aa+1] = ct:build_tbl()
@@ -1024,7 +1171,7 @@ function M.avail(argA)
    if (not quiet()) then
       aa = hook.apply("msgHook", "avail", aa)
    end
-   pcall(pager,io.stderr,concatTbl(aa,""))
+   shell:echo(concatTbl(aa,""))
    dbg.fini("Master.avail")
 end
 
