@@ -1,3 +1,33 @@
+--------------------------------------------------------------------------
+--  MasterControl is the base class for all the MC_Load, MC_Unload derived
+--  classes.  It has the Factory build member function as well as the
+--  functions that execute the commands from the modulefiles.  It may be
+--  helpful to understand the flow of control:
+--
+--    0. An MCP object is constructed (load, unload, show, help, etc)
+--    1. A modulefile is read and evaluated.
+--    2. A function in modfuncs is called.
+--    3. These modfuncs call a member function of the mcp object.
+--    4. All these member functions are implemented here.
+--
+--  So for example if the mcp object is a load version then
+--  when a modulefile calls setenv then the steps are:
+--    a) setenv(...) is called in modfuncs
+--    b) This calls mcp:setenv(...)
+--    c) the mcp load version connects it to MasterControl:setenv
+-- 
+--  Suppose a user is requesting to unload a module which contains a
+--  setenv command.
+--    0. An MCP unload object is constructed.
+--    1. The module file is read and evaluated
+--    2. The setenv function in modfuncs is called
+--    3. The unload MCP objects maps this to MasterControl:unsetenv
+--    4. The users' environment variable is unset.
+--
+-- @classmod MasterControl
+
+require("strict")
+
 --------------------------------------------------------------------------
 -- Lmod License
 --------------------------------------------------------------------------
@@ -32,37 +62,8 @@
 --
 --------------------------------------------------------------------------
 
---------------------------------------------------------------------------
---  MasterControl is the base class for all the MC_Load, MC_Unload derived
---  classes.  It has the Factory build member function as well as the
---  functions that execute the commands from the modulefiles.  It may be
---  helpful to understand the flow of control:
-
---  0) An MCP object is constructed (load, unload, show, help, etc)
---  1) A modulefile is read and evaluated.
---  2) A function in modfuncs is called.
---  3) These modfuncs call a member function of the mcp object.
---  4) All these member functions are implemented here.
-
---  So for example if the mcp object is a load version then
---  when a modulefile calls setenv then the steps are:
---  a) setenv(...) is called in modfuncs
---  b) This calls mcp:setenv(...)
---  c) the mcp load version connects it to MasterControl:setenv
-
---  Suppose a user is requesting to unload a module which contains a
---  setenv command.
---  0) An MCP unload object is constructed.
---  1) The module file is read and evaluated
---  2) The setenv function in modfuncs is called
---  3) The unload MCP objects maps this to MasterControl:unsetenv
---  4) The users' environment variable is unset.
---------------------------------------------------------------------------
-
--- MasterControl
-require("strict")
 require("TermWidth")
-require("escape")
+require("string_utils")
 require("inherits")
 require("utils")
 
@@ -81,25 +82,18 @@ local concatTbl    = table.concat
 local decode64     = base64.decode64
 local encode64     = base64.encode64
 local getenv       = os.getenv
+local remove       = table.remove
 local pack         = (_VERSION == "Lua 5.1") and argsPack or table.pack
+local Exit         = os.exit
+local s_moduleStk  = {}
+local s_loadT      = {}
 
-------------------------------------------------------------------------
---module ('MasterControl')
-------------------------------------------------------------------------
+--------------------------------------------------------------------------
+-- Check list of modules requested from user to see if they got loaded.
+-- If not found check with spider to see if a module can be loaded.
+function M.mustLoad()
 
-local function mustLoad(mA)
-   local aa = {}
-   local bb = {}
-
-   local mt = MT:mt()
-   for i = 1, #mA do
-      local mname = mA[i]
-      local sn    = mname:sn()
-      if (not mt:have(sn, "active")) then
-         aa[#aa+1] = mname:show()
-         bb[#bb+1] = mname:usrName()
-      end
-   end
+   local aa, bb = mcp.familyLoaded()
 
    if (#aa > 0) then
       local luaprog = "@path_to_lua@/lua"
@@ -110,7 +104,7 @@ local function mustLoad(mA)
       cmdA[#cmdA+1] = luaprog
       cmdA[#cmdA+1] = pathJoin(cmdDir(),cmdName())
       cmdA[#cmdA+1] = "bash"
-      cmdA[#cmdA+1] = "-r spider"
+      cmdA[#cmdA+1] = "-r --spider_timeout 2.0 spider"
       local count   = #cmdA
 
       local uA = {}  -- unknown names
@@ -118,17 +112,20 @@ local function mustLoad(mA)
       local kB = {}  -- known modules (usrName)
 
 
-
-      for i = 1, #bb do
-         cmdA[count+1] = "'^" .. escape(bb[i]) .. "$'"
-         cmdA[count+2] = "2> /dev/null"
-         local cmd     = concatTbl(cmdA," ")
-         local result  = capture(cmd)
-         if (result:find("\nfalse")) then
-            uA[#uA+1] = aa[i]
-         else
-            kA[#kA+1] = aa[i]
-            kB[#kB+1] = bb[i]
+      if (expert()) then
+         uA = aa
+      else
+         for i = 1, #bb do
+            cmdA[count+1] = "'^" .. bb[i]:escape() .. "$'"
+            cmdA[count+2] = "2> /dev/null"
+            local cmd     = concatTbl(cmdA," ")
+            local result  = capture(cmd)
+            if (result:find("\nfalse")) then
+               uA[#uA+1] = aa[i]
+            else
+               kA[#kA+1] = aa[i]
+               kB[#kB+1] = bb[i]
+            end
          end
       end
 
@@ -156,6 +153,9 @@ local function mustLoad(mA)
    end
 end
 
+--------------------------------------------------------------------------
+-- Build a list of user names based on mA.
+-- @param mA List of MName objects
 local function mAList(mA)
    local a = {}
    for i = 1, #mA do
@@ -164,22 +164,41 @@ local function mAList(mA)
    return concatTbl(a, ", ")
 end
 
+--------------------------------------------------------------------------
+-- Return the name of the derived MC object. 
+-- @param self A MasterControl object
 function M.name(self)
    return self.my_name
 end
 
+--------------------------------------------------------------------------
+-- Return the sType.
+-- @param self A MasterControl object
+function M.MNameType(self)
+   return self.my_sType
+end
+
+--------------------------------------------------------------------------
+-- Return the tcl_mode.
+-- @param self A MasterControl object
 function M.tcl_mode(self)
    return self.my_tcl_mode
 end
 
 
+--------------------------------------------------------------------------
+-- Convert MC name to MC Object.
+-- @param nameTbl Name to MC object table.
+-- @param name    Name of an MC objects.
 local function valid_name(nameTbl, name)
-   if (not nameTbl[name]) then
-      return nameTbl.default
-   end
-   return nameTbl[name]
+   return nameTbl[name] or nameTbl.default
 end
 
+--------------------------------------------------------------------------
+-- The Factory builder for the MasterControl Class.
+-- @param name the name of the derived object.
+-- @param[opt] mode An optional mode for building the *access* object.
+-- @return A derived MasterControl Object.
 function M.build(name,mode)
 
    local nameTbl          = {}
@@ -203,23 +222,31 @@ function M.build(name,mode)
 
    local o                = valid_name(nameTbl, name):create()
    o:_setMode(mode or name)
+   
 
    dbg.print{"Setting mcp to ", o:name(),"\n"}
    return o
 end
 
--------------------------------------------------------------------
--- Load / Unload functions
--------------------------------------------------------------------
-function M.load_usr(self, mA)
-   
 
+-------------------------------------------------------------------
+-- Load a list of modules.  Check to see if the user requested
+-- modules were actually loaded.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
+-- @return An array of statuses
+function M.load_usr(self, mA)
+   self.familyLoadRegister(mA)
    local a = self:load(mA)
-   if (haveWarnings()) then
-      mustLoad(mA)
-   end
    return a
 end
+
+-------------------------------------------------------------------
+-- Load a list of modules. Check to see if there are any admin
+-- messages.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
+-- @return An array of statuses
 function M.load(self, mA)
    LMOD_IGNORE_CACHE = true
    local master = Master:master()
@@ -280,6 +307,10 @@ function M.load(self, mA)
    return a
 end
 
+-------------------------------------------------------------------
+-- Load a list of module but ignore any warnings.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
 function M.try_load(self, mA)
    dbg.start{"MasterControl:try_load(mA)"}
    deactivateWarning()
@@ -287,6 +318,11 @@ function M.try_load(self, mA)
    dbg.fini("MasterControl:try_load")
 end
 
+-------------------------------------------------------------------
+-- Unload a list modules.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
+-- @return an array of statuses
 function M.unload(self, mA)
    local master = Master:master()
    local mt     = MT:mt()
@@ -297,10 +333,18 @@ function M.unload(self, mA)
    end
 
    local aa     = master.unload(mA)
-   dbg.fini("MasterControl:unload")
+   if (dbg.active()) then
+      dbg.fini("MasterControl:unload")
+   end
    return aa
 end
 
+-------------------------------------------------------------------
+-- Unload a user requested list of modules.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
+-- @param force if true then do not reload sticky modules.
+-- @return an array of statuses.
 function M.unload_usr(self, mA, force)
    dbg.start{"MasterControl:unload_usr(mA)"}
 
@@ -312,24 +356,17 @@ function M.unload_usr(self, mA, force)
 end
 
 
-function M.bad_unload(self,mA)
-   local a   = {}
-
-   dbg.start{"MasterControl.bad_unload(mA)"}
-
-   LmodWarning("Stubbornly refusing to unload module(s) during an unload\n")
-
-   dbg.fini("MasterControl.bad_unload")
-end
-
-
+-------------------------------------------------------------------
+-- This load is used by Manager Load to ignore load inside a
+-- module.
+-- @param self A MasterControl object
+-- @param mA A array of MName objects.
 function M.fake_load(self,mA)
-
    if (dbg.active()) then
       local s = mAList(mA)
       dbg.start{"MasterControl:fake_load(mA={"..s.."})"}
+      dbg.fini("MasterControl:fake_load")
    end
-   dbg.fini("MasterControl:fake_load")
 end
 
 
@@ -340,6 +377,12 @@ LMOD_MP_T = {}
 
 LMOD_MP_T[ModulePath]  = true
 LMOD_MP_T[DfltModPath] = true
+
+
+-------------------------------------------------------------------
+-- Prepend to a path like variable.
+-- @param self A MasterControl object
+-- @param t A table containing { name, value, nodups=v1, priority=v2}
 function M.prepend_path(self, t)
    dbg.start{"MasterControl:prepend_path(t)"}
    local sep      = t.delim or ":"
@@ -362,7 +405,10 @@ function M.prepend_path(self, t)
    dbg.fini("MasterControl:prepend_path")
 end
 
---function M.append_path(self, name, value, sep, nodups)
+--------------------------------------------------------------------------
+-- Append to a path like variable.
+-- @param self A MasterControl object
+-- @param t A table containing { name, value, nodups=v1, priority=v2}
 function M.append_path(self, t)
    local sep      = t.delim or ":"
    local name     = t[1]
@@ -385,6 +431,10 @@ function M.append_path(self, t)
    dbg.fini("MasterControl:append_path")
 end
 
+--------------------------------------------------------------------------
+-- Remove an entry from a path like variable.
+-- @param self A MasterControl object
+-- @param t A table containing { name, value, nodups=v1, priority=v2, where=v3}
 function M.remove_path(self, t)
    local sep      = t.delim or ":"
    local name     = t[1]
@@ -405,28 +455,39 @@ function M.remove_path(self, t)
    dbg.fini("MasterControl:remove_path")
 end
 
+--------------------------------------------------------------------------
+-- Remove an entry from a path-like variable.  This version is the reverse
+-- of a prepend_path.
+-- @param self A MasterControl object
+-- @param t A table containing { name, value, nodups=v1, priority=v2}
 function M.remove_path_first(self, t)
    t.where = "first"
    M.remove_path(self, t)
 end
 
+-- Remove an entry from a path-like variable.  This version is the reverse
+-- of a append_path.
+-- @param self A MasterControl object
+-- @param t A table containing { name, value, nodups=v1, priority=v2}
 function M.remove_path_last(self, t)
    t.where = "last"
    M.remove_path(self, t)
 end
 
 
-function M.bad_remove_path(self, t)
-   LmodWarning("Refusing remove a path element variable while unloading: \"",t[1],"\"\n")
-end
-
 -------------------------------------------------------------------
 -- Setenv / Unsetenv Functions
 -------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Set an environment variable.
+-- @param self A MasterControl object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
+-- @param respect If true, then respect the old value.
 function M.setenv(self, name, value, respect)
    dbg.start{"MasterControl:setenv(\"",name,"\", \"",value,"\", \"",
-              respect,")"}
+              respect,"\")"}
 
    if (respect and getenv(name)) then
       dbg.print{"Respecting old value"}
@@ -442,6 +503,12 @@ function M.setenv(self, name, value, respect)
    dbg.fini("MasterControl:setenv")
 end
 
+--------------------------------------------------------------------------
+-- Unset an environment variable.
+-- @param self A MasterControl object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
+-- @param respect If true, then respect the old value.
 function M.unsetenv(self, name, value, respect)
    dbg.start{"MasterControl:unsetenv(\"",name,"\", \"",value,"\")"}
 
@@ -458,14 +525,15 @@ function M.unsetenv(self, name, value, respect)
    dbg.fini("MasterControl:unsetenv")
 end
 
-function M.bad_unsetenv(self, name, value)
-   LmodWarning("Refusing unsetenv variable while unloading: \"",name,"\"\n")
-end
-
 -------------------------------------------------------------------
 -- stack: push and pop
 -------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Set an environment variable and remember previous values in a stack.
+-- @param self A MasterControl object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.pushenv(self, name, value)
    dbg.start{"MasterControl:pushenv(\"",name,"\", \"",value,"\")"}
 
@@ -502,6 +570,12 @@ function M.pushenv(self, name, value)
    dbg.fini("MasterControl:pushenv")
 end
 
+--------------------------------------------------------------------------
+-- The reverse action of pushenv.  It pops the old value off of the stack
+-- and set the *name* to the previous value from the stack.
+-- @param self A MasterControl object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.popenv(self, name, value)
    dbg.start{"MasterControl:popenv(\"",name,"\", \"",value,"\")"}
 
@@ -532,10 +606,12 @@ function M.popenv(self, name, value)
 end
 
 
--------------------------------------------------------------------
--- Alias Functions
--------------------------------------------------------------------
-
+--------------------------------------------------------------------------
+-- Set a shell alias.  This function can handle a single value for both
+-- bash and C-shell.
+-- @param self A MasterControl Object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.set_alias(self, name, value)
    dbg.start{"MasterControl:set_alias(\"",name,"\", \"",value,"\")"}
 
@@ -547,6 +623,11 @@ function M.set_alias(self, name, value)
    dbg.fini("MasterControl:set_alias")
 end
 
+--------------------------------------------------------------------------
+-- Unset a shell alias. 
+-- @param self A MasterControl Object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.unset_alias(self, name, value)
    dbg.start{"MasterControl:unset_alias(\"",name,"\", \"",value,"\")"}
 
@@ -557,14 +638,12 @@ function M.unset_alias(self, name, value)
    dbg.fini("MasterControl:unset_alias")
 end
 
-function M.bad_unset_alias(self, name)
-   LmodWarning("Refusing unset an alias while unloading: \"",name,"\"\n")
-end
-
--------------------------------------------------------------------
--- Shell Routine Functions
--------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Set a shell function for bash and a csh alias.
+-- @param self A MasterControl Object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.set_shell_function(self, name, bash_function, csh_function)
    dbg.start{"MasterControl:set_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
@@ -577,6 +656,11 @@ function M.set_shell_function(self, name, bash_function, csh_function)
    dbg.fini("MasterControl:set_shell_function")
 end
 
+--------------------------------------------------------------------------
+-- Unset a shell function for bash and a csh alias.
+-- @param self A MasterControl Object.
+-- @param name the environment variable name.
+-- @param value the environment variable value.
 function M.unset_shell_function(self, name, bash_function, csh_function)
    dbg.start{"MasterControl:unset_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
@@ -588,14 +672,15 @@ function M.unset_shell_function(self, name, bash_function, csh_function)
    dbg.fini("MasterControl:unset_shell_function")
 end
 
-function M.bad_unset_shell_function(self, name, bash_function, csh_function)
-   LmodWarning("Refusing unset a shell function while unloading: \"",name,"\"\n")
-end
-
 -------------------------------------------------------------------
 -- Property Functions
 -------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Set a property value
+-- @param self A MasterControl Object.
+-- @param name A property name
+-- @param value A property value.
 function M.add_property(self, name, value)
    local mStack  = ModuleStack:moduleStack()
    local mFull   = mStack:fullName()
@@ -604,6 +689,11 @@ function M.add_property(self, name, value)
    mt:add_property(mname:sn(), name, value)
 end
 
+--------------------------------------------------------------------------
+-- Unset a property value
+-- @param self A MasterControl Object.
+-- @param name A property name
+-- @param value A property value.
 function M.remove_property(self, name, value)
    local mStack  = ModuleStack:moduleStack()
    local mFull   = mStack:fullName()
@@ -612,14 +702,9 @@ function M.remove_property(self, name, value)
    mt:remove_property(mname:sn(), name, value)
 end
 
-function M.bad_remove_property(self, name, value)
-   LmodWarning("Refusing to remove a property while unloading: \"",name,"\"\n")
-end
-
--------------------------------------------------------------------
--- Message/Error  Functions
--------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Report the modulefiles stack for error report.
 local function moduleStackTraceBack()
    local mStack = ModuleStack:moduleStack()
    if (mStack:empty()) then return end
@@ -638,24 +723,15 @@ local function moduleStackTraceBack()
                    bt:build_tbl(),"\n")
 end
 
-local function msg(lead, ...)
-   local twidth = TermWidth()
-   local arg = { n = select('#', ...), ...}
-   io.stderr:write("\n")
-   arg[1] = lead .. arg[1]
-   local whole = concatTbl(arg,"")
-   for s in whole:split("\n") do
-      io.stderr:write(fillWords("",s, twidth),"\n")
-   end
-   io.stderr:write("\n")
-   moduleStackTraceBack()
-end
-
+--------------------------------------------------------------------------
+-- Write "false" to stdout and exit.
 function LmodErrorExit()
    io.stdout:write("false\n")
-   os.exit(1)
+   Exit(1)
 end
 
+--------------------------------------------------------------------------
+-- Print msgs, traceback then exit.
 function LmodSystemError(...)
    io.stderr:write("\n", colorize("red", "Lmod has detected the following error: "))
    local arg = pack(...)
@@ -668,10 +744,16 @@ function LmodSystemError(...)
 end
 
 
+--------------------------------------------------------------------------
+-- Print msgs, traceback then exit.
+-- @param self A MasterControl object.
 function M.error(self, ...)
    LmodSystemError(...)
 end
 
+--------------------------------------------------------------------------
+-- Print msgs, traceback then set warning flag.
+-- @param self A MasterControl object.
 function M.warning(self, ...)
    if (not quiet() and  haveWarnings()) then
       io.stderr:write("\n",colorize("red", "Lmod Warning: "))
@@ -686,6 +768,9 @@ function M.warning(self, ...)
 end
 
 
+--------------------------------------------------------------------------
+-- Print msgs to stderr.
+-- @param self A MasterControl object.
 function M.message(self, ...)
    if (quiet()) then
       return
@@ -698,10 +783,11 @@ function M.message(self, ...)
 end
 
 
--------------------------------------------------------------------
--- Misc Functions
--------------------------------------------------------------------
 
+--------------------------------------------------------------------------
+-- Check the prereq from *mA*.
+-- @param self A MasterControl object.
+-- @param mA An array of MNname objects.
 function M.prereq(self, mA)
    local mt        = MT:mt()
    local a         = {}
@@ -734,6 +820,10 @@ function M.prereq(self, mA)
    dbg.fini("MasterControl:prereq")
 end
 
+--------------------------------------------------------------------------
+-- Check the conflicts from *mA*.
+-- @param self A MasterControl object.
+-- @param mA An array of MNname objects.
 function M.conflict(self, mA)
    dbg.start{"MasterControl:conflict(mA)"}
 
@@ -774,6 +864,11 @@ function M.conflict(self, mA)
    dbg.fini("MasterControl:conflict")
 end
 
+--------------------------------------------------------------------------
+-- Check the prereq from *mA*.  If any of them are acceptable then return.
+-- otherwise error out.
+-- @param self A MasterControl object.
+-- @param mA An array of MNname objects.
 function M.prereq_any(self, mA)
    local mt        = MT:mt()
    local a         = {}
@@ -813,7 +908,91 @@ function M.prereq_any(self, mA)
 end
 
 
+--------------------------------------------------------------------------
+-- Remember the user's requested load array into an internal table.
+-- This is tricky because the module mnames in the *mA* array may not be
+-- findable yet (e.g. module load mpich petsc).  The only thing we know
+-- is the usrName from the command line.  So we use the *usrName* to be
+-- the key and not *sn*.
+-- @param mA The array of MName objects.
+function M.familyLoadRegister(mA)
+   dbg.start{"familyLoadRegister(mA)"}
+   for i = 1, #mA do
+      local mname      = mA[i]
+      local usrName    = mname:usrName()
+      s_loadT[usrName] = mname
+      dbg.print{"usrName: ",usrName,"\n"}
+   end
 
+   dbg.fini("familyLoadRegister")
+end
+
+--------------------------------------------------------------------------
+-- Check the currently loaded table modules to see if any didn't get
+-- loaded.
+-- @return An array of descripted name of missing modules.
+-- @return An array of the user names of missing modules.
+function M.familyLoaded()
+   dbg.start{"MC:familyLoaded()"}
+   local mt        = MT:mt()
+
+   local aa = {}
+   local bb = {}
+
+   for usrName, mname in pairs(s_loadT) do
+      local sn = mname:sn()
+      dbg.print{"usrName: ",usrName, " is ", not not mt:have(sn, "active"), "\n"}
+      if (not mt:have(sn, "active")) then
+         aa[#aa+1] = mname:show()
+         bb[#bb+1] = usrName
+      end
+   end
+         
+   dbg.fini("MC:familyLoaded")
+   return aa, bb
+end
+
+------------------------------------------------------------------------
+-- Save away the modules that are in the same family.
+-- @param oldName The old module name that is getting pushed out by *sn*.
+-- @param sn The new module name.
+function M.familyStackPush(oldName, sn)
+   dbg.start{"familyStackPush(",oldName,", ", sn,")"}
+   local mt             = MT:mt()
+   local old_usrName    = mt:usrName(oldName)
+   dbg.print{"removing old sn: ",oldName,",old usrName: ",old_usrName,"\n"}
+
+   s_loadT[old_usrName] = nil
+   s_moduleStk[#s_moduleStk+1] = { sn=oldName, fullName = mt:fullName(oldName),
+                                   usrName = mt:usrName(oldName)}
+   s_moduleStk[#s_moduleStk+1] = { sn=sn,      fullName = mt:fullName(sn),
+                                   usrName = mt:usrName(sn)}
+   dbg.fini("familyStackPush")
+end
+
+--------------------------------------------------------------------------
+-- Pop the top two value off the stack.
+-- @return the top two value on the stack.
+function M.familyStackPop()
+   local valueN = s_moduleStk[#s_moduleStk]
+   remove(s_moduleStk)
+   local valueO = s_moduleStk[#s_moduleStk]
+   remove(s_moduleStk)
+   return valueO, valueN
+end
+
+--------------------------------------------------------------------------
+-- Check for an empty stack.
+-- @return True if the stack is empty.
+function M.familyStackEmpty()
+   return (next(s_moduleStk) == nil)
+end
+
+--------------------------------------------------------------------------
+-- Process the family function.  The name of the module is found by the
+-- *ModuleStack*.
+-- @param self A MasterControl object
+-- @param name The name of the family
 function M.family(self, name)
    local mt        = MT:mt()
    local mStack    = ModuleStack:moduleStack()
@@ -830,14 +1009,20 @@ function M.family(self, name)
    end
 
    dbg.print{"mt:setfamily(\"",name,"\",\"",sn,"\")\n"}
-   local oldName = mt:setfamily(name,sn)
+   local oldName = mt:getfamily(name)
    if (oldName ~= nil and oldName ~= sn and not expert() ) then
-      LmodError("You can only have one ",name," module loaded at a time.\n",
-                "You already have ", oldName," loaded.\n",
-                "To correct the situation, please enter the following command:\n\n",
-                "  module swap ",oldName, " ", mFull,"\n\n",
-                "Please submit a consulting ticket if you require additional assistance.\n")
+      if (LMOD_AUTO_SWAP ~= "no") then
+         dbg.print{"RTM Fmly Push sn: ",sn,", oldName: ",oldName,"\n"}
+         self.familyStackPush(oldName, sn)
+      else
+         LmodError("You can only have one ",name," module loaded at a time.\n",
+                   "You already have ", oldName," loaded.\n",
+                   "To correct the situation, please enter the following command:\n\n",
+                   "  module swap ",oldName, " ", mFull,"\n\n",
+                   "Please submit a consulting ticket if you require additional assistance.\n")
+      end
    end
+   mt:setfamily(name,sn)
    dbg.fini("MasterControl:family")
 end
 
@@ -927,6 +1112,8 @@ function M.execute(self, t)
 end
 
 
+
+
 -------------------------------------------------------------------
 -- Quiet Functions
 -------------------------------------------------------------------
